-- module_mrm.lua: Mission Replay Mini [nclok1405]

MRM_MISSIONS = {
	{ mission_name = "m01", display_name = "MISSION_NAME_M01" },
	{ mission_name = "m02", display_name = "MISSION_NAME_M02" },
	{ mission_name = "m03", display_name = "MISSION_NAME_M03" },
	{ mission_name = "m04", display_name = "MISSION_NAME_M04" },
	{ mission_name = "m05", display_name = "MISSION_NAME_M05" },
	{ mission_name = "mm_p_03", display_name = "MISSION_NAME_MM_P_03" },
	{ mission_name = "sh01", display_name = "MISSION_NAME_SH01" },
	{ mission_name = "m06", display_name = "MISSION_NAME_M06" },
	{ mission_name = "m07", display_name = "MISSION_NAME_M07" },
	{ mission_name = "m08", display_name = "MISSION_NAME_M08" },
	{ mission_name = "sh02", display_name = "MISSION_NAME_SH02" },
	{ mission_name = "m09", display_name = "MISSION_NAME_M09" },
	{ mission_name = "m10", display_name = "MISSION_NAME_M10" },
	{ mission_name = "m11", display_name = "MISSION_NAME_M11" },
	{ mission_name = "m12", display_name = "MISSION_NAME_M12" },
	{ mission_name = "m13", display_name = "MISSION_NAME_M13" },
	{ mission_name = "m14", display_name = "MISSION_NAME_M14" },
	{ mission_name = "m15", display_name = "MISSION_NAME_M15" },
	{ mission_name = "sh03", display_name = "MISSION_NAME_SH03" },
	{ mission_name = "m16", display_name = "MISSION_NAME_M16" },
	{ mission_name = "m17", display_name = "MISSION_NAME_M17" },
	{ mission_name = "m18", display_name = "MISSION_NAME_M18" },
	{ mission_name = "m19", display_name = "MISSION_NAME_M19" },
	{ mission_name = "m20", display_name = "MISSION_NAME_M20" },
	{ mission_name = "sh04", display_name = "MISSION_NAME_SH04" },
	{ mission_name = "m21", display_name = "MISSION_NAME_M21" },
	{ mission_name = "m22", display_name = "MISSION_NAME_M22" },
	{ mission_name = "m23", display_name = "MISSION_NAME_M23" },
	{ mission_name = "m24", display_name = "MISSION_NAME_M24" },
	{ mission_name = "dlc2_m01", display_name = "DLC2_M01_NAME" },
	{ mission_name = "dlc2_m02", display_name = "DLC2_M02_MISSION_NAME" },
	{ mission_name = "dlc2_m03", display_name = "DLC2_M03_MISSION_NAME" },
	{ mission_name = "dlc3_m01", display_name = "DLC3_M01_MISSION_NAME" },
	{ mission_name = "dlc3_m02", display_name = "DLC3_M02_MISSION_NAME" },
	{ mission_name = "dlc3_m03", display_name = "DLC3_M03_MISSION_NAME" },
}

Mrm_mission_table = {}
Mrm_marker_active = false
Mrm_prompt_active = false
Mrm_help_displayed = false
Mrm_selected_mission_number = 1

function mrm_main_thread()
	-- Wait until the loading screen goes away
	fade_in_block()
	
	-- Wait until the player completes the prologue
	while (not mission_is_complete("m03")) or mission_is_active() do
		thread_yield()
	end
	
	-- Setup the completed mission list
	for i, mission in pairs(MRM_MISSIONS) do
		if mission_is_complete(mission.mission_name) then
			Mrm_mission_table[#Mrm_mission_table + 1] = mission
		end
	end
	
	while true do
		thread_yield()
		
		-- If mission becomes active, disable everything and wait until it's over
		if mission_is_active() then
			if Mrm_marker_active then
				marker_remove("m11", SYNC_LOCAL)
				Mrm_marker_active = false
			end
			if Mrm_prompt_active then
				hud_prompt_clear(LOCAL_PLAYER)
				Mrm_prompt_active = false
			end
			
			-- Restore Police Notoriety
			notoriety_set_min("police", 0)
			notoriety_set_max("police", 5)

			-- Wait until the mission is no longer active
			while mission_is_active() do
				thread_yield()
			end
			
			-- Setup the completed mission list again, so when a new mission is completed it is added to the replay
			Mrm_mission_table = {}
			for i, mission in pairs(MRM_MISSIONS) do
				if mission_is_complete(mission.mission_name) then
					Mrm_mission_table[#Mrm_mission_table + 1] = mission
				end
			end
		end
		
		-- Marker
		if (not Mrm_marker_active) and mrm_is_player_close_enough(20) then
			marker_add("m11", "map_other_replaceme", OI_ASSET_USE, OI_FLAGS_FULL, SYNC_LOCAL)
			
			-- Lock Police Notoriety to 0 so STAG don't annoy you
			notoriety_set_min("police", 0)
			notoriety_set_max("police", 0)
			notoriety_set("police", 0)

			Mrm_marker_active = true
		elseif Mrm_marker_active and (not mrm_is_player_close_enough(20)) then
			marker_remove("m11", SYNC_LOCAL)
			
			-- Restore Police Notoriety
			notoriety_set_min("police", 0)
			notoriety_set_max("police", 5)
			
			Mrm_marker_active = false
		end
		
		-- Show Prompt
		if (not Mrm_prompt_active) and mrm_is_player_close_enough(5) then
			hud_prompt(LOCAL_PLAYER, Mrm_mission_table[Mrm_selected_mission_number].display_name, "mrm_triggered")
			
			-- Help message
			if not Mrm_help_displayed then
				message("PUSH {AUDIO_PLAYER_PREV_TRACK_IMG} or {AUDIO_PLAYER_NEXT_TRACK_IMG} TO SELECT MISSIONS", 5, false, SYNC_LOCAL)
				Mrm_help_displayed = true
			end
			
			-- Restore completed strongholds to Saints control.
			-- I don't know an accurate way to detect mission exit.
			-- Dying causes mission_is_active() to report false even if we retry from a checkpoint.
			if mission_is_complete("sh02") then
				stronghold_magic("safeword", true)
			end
			if mission_is_complete("sh03") then
				stronghold_magic("nuke", true)
			end
			if mission_is_complete("sh04") then
				stronghold_magic("3count", true)
			end

			Mrm_prompt_active = true
		elseif Mrm_prompt_active and (not mrm_is_player_close_enough(5)) then
			hud_prompt_clear(LOCAL_PLAYER)
			Mrm_prompt_active = false
		end
		
		-- Next
		if Mrm_prompt_active and (player_action_just_pressed("CBA_OFC_PICKUP_RELOAD") or player_action_just_pressed("CBA_VDC_NEXT_RADIO")) then
			hud_prompt_clear(LOCAL_PLAYER)
			
			Mrm_selected_mission_number = Mrm_selected_mission_number + 1
			
			if Mrm_selected_mission_number > #Mrm_mission_table then
				Mrm_selected_mission_number = 1
			end
			
			hud_prompt(LOCAL_PLAYER, Mrm_mission_table[Mrm_selected_mission_number].display_name, "mrm_triggered")
		end
		
		-- Previous
		if Mrm_prompt_active and player_action_just_pressed("CBA_VDC_PREV_RADIO") then
			hud_prompt_clear(LOCAL_PLAYER)
			
			Mrm_selected_mission_number = Mrm_selected_mission_number - 1
			
			if Mrm_selected_mission_number < 1 then
				Mrm_selected_mission_number = #Mrm_mission_table
			end
			
			hud_prompt(LOCAL_PLAYER, Mrm_mission_table[Mrm_selected_mission_number].display_name, "mrm_triggered")
		end
		
	end
	
end

-- Check if the player is close enough to the trigger
function mrm_is_player_close_enough(dist)
	local px, py, pz = get_object_pos(LOCAL_PLAYER)
	local ox, oy, oz = 97.9103, 8.1554, 1017.2667
	
	local dx = px - ox
	local dy = py - oy
	local dz = pz - oz
	
	if dx < 0 then
		dx = -dx
	end
	if dy < 0 then
		dy = -dy
	end
	if dz < 0 then
		dz = -dz
	end

	if (dx < dist) and (dy < dist) and (dz < dist) then
		return true
	end
	
	return false
end

-- Trigger activated
function mrm_triggered()
	if Mrm_marker_active then
		marker_remove("m11", SYNC_LOCAL)
		Mrm_marker_active = false
	end
	if Mrm_prompt_active then
		hud_prompt_clear(LOCAL_PLAYER)
		Mrm_prompt_active = false
	end
	
	-- Restore Police Notoriety
	notoriety_set_min("police", 0)
	notoriety_set_max("police", 5)

	local mission_name = Mrm_mission_table[Mrm_selected_mission_number].mission_name
	--message(mission_name, 5, false, SYNC_LOCAL)
	
	-- Change world state based on the selected mission (world_state_magic and stronghold_magic are handled by sr3_city.lua)
	if mission_name == "m02" then
		world_state_magic("m01_05")
	elseif mission_name == "m03" then
		world_state_magic("m01_05")
	elseif mission_name == "m04" then
		world_state_magic("m01_05")
	elseif mission_name == "m05" then
		world_state_magic("m01_05")
	elseif mission_name == "m06" then
		world_state_magic("m06")
	elseif mission_name == "m07" then
		world_state_magic("m07_10")
		
		-- Rescue Zimos requires Safeword stronghold
		if mission_is_complete("sh02") then
			stronghold_magic("safeword", false)
		end
	elseif mission_name == "m08" then
		world_state_magic("m07_10")
		
		-- End Cutscene needs Nuke Plant
		if mission_is_complete("sh03") then
			stronghold_magic("nuke", false)
		end
	elseif mission_name == "m09" then
		world_state_magic("m07_10")
		
		-- End Cutscene needs 3count
		if mission_is_complete("sh04") then
			stronghold_magic("3count", false)
		end
	elseif mission_name == "m10" then
		world_state_magic("m07_10")
	elseif mission_name == "m11" then
		world_state_magic("m11_13")
	elseif mission_name == "m12" then
		world_state_magic("m11_13")
	elseif mission_name == "m13" then
		world_state_magic("m11_13")
	elseif mission_name == "m14" then
		world_state_magic("m14")
	elseif mission_name == "m15" then
		world_state_magic("m15_17")
	elseif mission_name == "m16" then
		world_state_magic("m15_17")
		
		-- End Cutscene needs 3count
		if mission_is_complete("sh04") then
			stronghold_magic("3count", false)
		end
	elseif mission_name == "m17" then
		world_state_magic("m15_17")
	elseif mission_name == "m18" then
		world_state_magic("m18")
	elseif mission_name == "m19" then
		world_state_magic("m19")
	elseif mission_name == "m20" then
		world_state_magic("m20")
	elseif mission_name == "m21" then
		world_state_magic("m21_22")
	elseif mission_name == "m22" then
		world_state_magic("m21_22")
	elseif mission_name == "m23" then
		world_state_magic("m23")
	elseif mission_name == "m24" then
		world_state_magic("final")
	elseif mission_name == "sh01" then
		world_state_magic("m01_06")
	elseif mission_name == "sh02" then
		world_state_magic("sh02")
		stronghold_magic("safeword", false)
	elseif mission_name == "sh03" then
		world_state_magic("sh03")
		stronghold_magic("nuke", false)
	elseif mission_name == "sh04" then
		world_state_magic("sh04")
		stronghold_magic("3count", false)
	end
	
	-- Warp to the mission start (if we don't do this the game softlocks at the infinite loading screen)
	sr3_teleport_mission_start(mission_name, false)
	
	-- Begin the mission
	mission_set_next_mission(mission_name)
end
